"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BullDataSource = void 0;
const apollo_datasource_1 = require("apollo-datasource");
const json_1 = require("../../services/json");
const gql_1 = require("../../typings/gql");
const redis_info_1 = __importDefault(require("redis-info"));
class BullDataSource extends apollo_datasource_1.DataSource {
    constructor(_queues) {
        super();
        this._queues = _queues;
        this._queuesMap = new Map();
        this._convertQueuesToMap(_queues);
    }
    _convertQueuesToMap(queues) {
        queues.forEach(queue => {
            this._queuesMap.set(queue.name, queue);
        });
    }
    getQueueByName(name) {
        return this._queuesMap.get(name);
    }
    getQueues() {
        return this._queues;
    }
    createJob({ queue: queueName, name = null, data = {}, options = {}, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this._queuesMap.get(queueName);
            return yield (queue === null || queue === void 0 ? void 0 : queue.add(name, json_1.JsonService.maybeParse(data), json_1.JsonService.maybeParse(options)));
        });
    }
    getQueueJobs({ queue, limit = 20, offset = 0, status = [], id, order = gql_1.OrderEnum.Desc, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const bullQueue = this.getQueueByName(queue);
            if (id) {
                const job = yield (bullQueue === null || bullQueue === void 0 ? void 0 : bullQueue.getJob(id));
                return job ? [job] : [];
            }
            return bullQueue === null || bullQueue === void 0 ? void 0 : bullQueue.getJobs(status, offset, offset + limit - 1, order === gql_1.OrderEnum.Asc);
        });
    }
    getJob(queueName, id) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(queueName);
            return queue === null || queue === void 0 ? void 0 : queue.getJob(id);
        });
    }
    getJobLogs(queueName, id) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(queueName);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getJobLogs(id));
        });
    }
    createJobLog(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id);
            yield (job === null || job === void 0 ? void 0 : job.log(args.row));
            return job;
        });
    }
    retryJob(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id);
            yield (job === null || job === void 0 ? void 0 : job.retry());
            return job;
        });
    }
    removeJobById(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id);
            yield (job === null || job === void 0 ? void 0 : job.remove());
            return job;
        });
    }
    moveJobToCompleted(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id);
            yield (job === null || job === void 0 ? void 0 : job.moveToCompleted());
            return job;
        });
    }
    moveJobToFailed(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id);
            yield (job === null || job === void 0 ? void 0 : job.moveToFailed({
                message: '',
            }));
            return job;
        });
    }
    promoteJob(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id);
            yield (job === null || job === void 0 ? void 0 : job.promote());
            return job;
        });
    }
    discardJob(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id);
            yield (job === null || job === void 0 ? void 0 : job.discard());
            return job;
        });
    }
    updateJobData(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = yield this.getJob(args.queue, args.id);
            yield (job === null || job === void 0 ? void 0 : job.update(json_1.JsonService.maybeParse(args.data)));
            return job;
        });
    }
    getQueueJobsCounts(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.getJobCounts());
        });
    }
    pauseQueue(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            yield (queue === null || queue === void 0 ? void 0 : queue.pause());
            return queue;
        });
    }
    cleanQueue(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue);
            return yield (queue === null || queue === void 0 ? void 0 : queue.clean(args.grace, args.status));
        });
    }
    emptyQueue(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue);
            yield (queue === null || queue === void 0 ? void 0 : queue.empty());
            return queue;
        });
    }
    closeQueue(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue);
            yield (queue === null || queue === void 0 ? void 0 : queue.close());
            return queue;
        });
    }
    resumeQueue(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue);
            yield (queue === null || queue === void 0 ? void 0 : queue.resume());
            return queue;
        });
    }
    getQueueWaitingOrDelayedJobsCount(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(name);
            return yield (queue === null || queue === void 0 ? void 0 : queue.count());
        });
    }
    removeJobsByPattern(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const queue = this.getQueueByName(args.queue);
            yield (queue === null || queue === void 0 ? void 0 : queue.removeJobs(args.pattern));
            return true;
        });
    }
    getRedisInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._queuesMap.size > 0) {
                const firstQueue = this._queues[0];
                const rawInfo = yield firstQueue.client.info();
                return redis_info_1.default.parse(rawInfo);
            }
            return null;
        });
    }
}
exports.BullDataSource = BullDataSource;
//# sourceMappingURL=bull.js.map